@@ -7,6 +7,8 @@
 #' @param multiCV Logical. Re-run model fitting and validation with different CV strategies. See details.
 #' @param length.out Numeric. Only used if multiCV=TRUE. Number of cross-validation folds. See details.
 #' @param maskAOA Logical. Should areas outside the AOA set to NA?
+#' @param method Character. Method used for distance calculation. Currently euclidean distance (L2) and Mahalanobis distance (MD) are implemented but only L2 is tested. Note that MD takes considerably longer. See ?aoa for further explanation
+#' @param useWeight Logical. Only if a model is given. Weight variables according to importance in the model?
 #' @param k Numeric. See mgcv::s
 #' @param m Numeric. See mgcv::s
 #' @param showPlot Logical.
@@ -62,24 +64,27 @@
 #' @aliases calibrate_aoa
 
 calibrate_aoa <- function(AOA,model, window.size=5, calib="scam",multiCV=FALSE,
-                          length.out = 10, maskAOA=TRUE, showPlot=TRUE,k=6,m=2){
+                          length.out = 10, maskAOA=TRUE, method= "L2", useWeight=TRUE,
+                          showPlot=TRUE,k=6,m=2){
 
   as_stars <- FALSE
   as_terra <- FALSE
-  if (inherits(AOA, "stars")) {
+  if (inherits(AOA$AOA, "stars")) {
     if (!requireNamespace("stars", quietly = TRUE))
       stop("package stars required: install that first")
     attr <- attributes(AOA)[c("aoa_stats","TrainDI")]
-    AOA <- methods::as(AOA, "Raster")
+    AOA$AOA <- methods::as(AOA$AOA, "Raster")
+    AOA$DI <- methods::as(AOA$DI, "Raster")
     attributes(AOA)<- c(attributes(AOA),attr)
     as_stars <- TRUE
   }
 
-  if (inherits(AOA, "SpatRaster")) {
+  if (inherits(AOA$AOA, "SpatRaster")) {
     if (!requireNamespace("terra", quietly = TRUE))
       stop("package terra required: install that first")
     attr <- attributes(AOA)[c("aoa_stats","TrainDI")]
-    AOA <- methods::as(AOA, "Raster")
+    AOA$AOA <- methods::as(AOA$AOA, "Raster")
+    AOA$DI <- methods::as(AOA$DI, "Raster")
     attributes(AOA)<- c(attributes(AOA),attr)
     as_terra <- TRUE
   }
@@ -104,10 +109,13 @@ calibrate_aoa <- function(AOA,model, window.size=5, calib="scam",multiCV=FALSE,
       mcall$y <- quote(train_response)
       mcall$trControl <- trainControl(method="cv",index=folds$index,savePredictions = TRUE)
       mcall$tuneGrid <- model$bestTune
+      mcall$method <- model$method
+      mcall$metric <- model$metric
+      mcall$cl <- NULL # fix option for parallel later
 
       # retrain model and calculate AOA
       model_new <- do.call(caret::train,mcall)
-      AOA_new <- aoa(train_predictors,model_new)
+      AOA_new <- aoa(train_predictors,model_new,method=method, useWeight=useWeight)
 
       # legacy change (very dirty, change this as soon as possible)
       #AOA_new <- AOA_new$AOA
@@ -130,6 +138,9 @@ calibrate_aoa <- function(AOA,model, window.size=5, calib="scam",multiCV=FALSE,
   ### Get cross-validated predictions from the model:
   if(!multiCV){
     # Get cross-validated predictions:
+    if(is.null(model$pred)){
+      stop("CV predictions cannot be derived from the model. re-train using savePredictions, see ?caret::trainControl")
+    }
     preds_all <- model$pred
     for (i in 1:length(model$bestTune)){
       tunevar <- names(model$bestTune[i])
@@ -137,7 +148,7 @@ calibrate_aoa <- function(AOA,model, window.size=5, calib="scam",multiCV=FALSE,
     }
 
     preds_all <- preds_all[order(preds_all$rowIndex),c("pred","obs")]
-    preds_all$DI <- AOA$parameters$trainDI
+    preds_all$DI <- AOA$parameters$trainDI[!is.na(AOA$parameters$trainDI)]
 
     ## only take predictions from inside the AOA:
     preds_all <-  preds_all[preds_all$DI<=AOA$parameters$threshold,]
@@ -291,13 +302,15 @@ calibrate_aoa <- function(AOA,model, window.size=5, calib="scam",multiCV=FALSE,
 
 
   if (as_stars){
-    AOA <- split(stars::st_as_stars(AOA), "band")
-    attributes(AOA)<- c(attributes(AOA),attr)
+    AOA$AOA <- split(stars::st_as_stars(AOA$AOA), "band")
+    AOA$DI <- split(stars::st_as_stars(AOA$DI), "band")
+    attributes(AOA$AOA)<- c(attributes(AOA$AOA),attr)
   }
 
   if(as_terra){
-    AOA <- methods::as(AOA, "SpatRaster")
-    attributes(AOA)<- c(attributes(AOA),attr)
+    AOA$AOA <- methods::as(AOA$AOA, "SpatRaster")
+    AOA$DI <- methods::as(AOA$DI, "SpatRaster")
+    attributes(AOA$AOA)<- c(attributes(AOA$AOA),attr)
   }
   names(AOA)[names(AOA)=="expectedError"] <- paste0("expected_",model$metric)
   #return(AOA)
@@ -305,7 +318,6 @@ calibrate_aoa <- function(AOA,model, window.size=5, calib="scam",multiCV=FALSE,
   return(list(AOA = AOA,
               plot = p))
 
-
 }
 
 
