@@ -14,7 +14,7 @@ library(ggplot2)
 seed <- 10 # random realization
 samplesize <- 300 # how many samples will be used?
 nparents <- 20 #For clustered samples: How many clusters? 
-radius <- 7 # For clustered samples: What is the radius of a cluster?
+radius <- 500000 # For clustered samples: What is the radius of a cluster?
 
 
 ## ----message = FALSE, warning=FALSE-------------------------------------------
@@ -25,7 +25,7 @@ co.ee <- st_transform(co, ee)
 ## ----message = FALSE, warning=FALSE, results='hide'---------------------------
 sf_use_s2(FALSE)
 set.seed(seed)
-pts_random <- st_sample(co, samplesize)
+pts_random <- st_sample(co.ee, samplesize)
 ### See points on the map:
 ggplot() + geom_sf(data = co.ee, fill="#00BFC4",col="#00BFC4") +
   geom_sf(data = pts_random, color = "#F8766D",size=0.5, shape=3) +
@@ -33,38 +33,10 @@ ggplot() + geom_sf(data = co.ee, fill="#00BFC4",col="#00BFC4") +
   labs(x = NULL, y = NULL)
 
 
-## ----message = FALSE, include=FALSE-------------------------------------------
-# adjusted from from https://github.com/carlesmila/NNDMpaper/blob/main/code/sim_utils.R
-clustered_sample <- function(sarea, nsamples, nparents, radius){
-
-  # Number of offspring per parent
-  nchildren <- round((nsamples-nparents)/nparents, 0)
-
-  # Simulate parents
-  parents <- st_sf(geometry=st_sample(sarea, nparents, type="random"))
-  res <- parents
-  res$parent <- 1:nrow(parents)
-
-  # Simulate offspring
-  for(i in 1:nrow(parents)){
-
-    # Generate buffer and cut parts outside of the area of study
-    buf <- st_buffer(parents[i,], dist=radius)
-    buf <- st_intersection(buf, sarea)
-
-    # Simulate children
-    children <- st_sf(geometry=st_sample(buf, nchildren, type="random"))
-      children$parent <- i
-    res <- rbind(res, children)
-  }
-
-  return(res)
-}
-
 ## ----message = FALSE, warning=FALSE, results='hide'---------------------------
 set.seed(seed)
 sf_use_s2(FALSE)
-pts_clustered <- clustered_sample(co, samplesize, nparents, radius)
+pts_clustered <- clustered_sample(co.ee, samplesize, nparents, radius)
 
 ggplot() + geom_sf(data = co.ee, fill="#00BFC4",col="#00BFC4") +
   geom_sf(data = pts_clustered, color = "#F8766D",size=0.5, shape=3) +
@@ -73,11 +45,11 @@ ggplot() + geom_sf(data = co.ee, fill="#00BFC4",col="#00BFC4") +
 
 
 ## ----message = FALSE, warning=FALSE, results='hide'---------------------------
-dist_random <- plot_geodist(pts_random,co,
+dist_random <- plot_geodist(pts_random,co.ee,
                             sampling="Fibonacci",
                             unit="km",
                             showPlot = FALSE)
-dist_clstr <- plot_geodist(pts_clustered,co,
+dist_clstr <- plot_geodist(pts_clustered,co.ee,
                            sampling="Fibonacci",
                            unit="km",
                            showPlot = FALSE)
@@ -100,7 +72,7 @@ ggplot() + geom_sf(data = co.ee, fill="#00BFC4",col="#00BFC4") +
   labs(x = NULL, y = NULL)+ggtitle("random fold membership shown by color")
 
 ## ----message = FALSE, warning=FALSE, results='hide'---------------------------
-dist_clstr <- plot_geodist(pts_clustered,co,
+dist_clstr <- plot_geodist(pts_clustered,co.ee,
                            sampling="Fibonacci", 
                            cvfolds= randomfolds, 
                            unit="km",
@@ -118,7 +90,7 @@ ggplot() + geom_sf(data = co.ee, fill="#00BFC4",col="#00BFC4") +
   labs(x = NULL, y = NULL)+ ggtitle("spatial fold membership by color")
 
 ## ----message = FALSE, warning=FALSE, results='hide'---------------------------
-dist_clstr <- plot_geodist(pts_clustered,co,
+dist_clstr <- plot_geodist(pts_clustered,co.ee,
                            sampling="Fibonacci",
                            cvfolds= spatialfolds$indexOut, 
                            unit="km",
@@ -131,7 +103,7 @@ dist_clstr$plot+scale_x_log10(labels=round)
 # create a spatial CV for the randomly distributed data. Here:
 # "leave region-out-CV"
 sf_use_s2(FALSE)
-pts_random_co <- st_join(st_as_sf(pts_random),co)
+pts_random_co <- st_join(st_as_sf(pts_random),co.ee)
 
 
 ggplot() + geom_sf(data = co.ee, fill="#00BFC4",col="#00BFC4") +
@@ -145,7 +117,7 @@ ggplot() + geom_sf(data = co.ee, fill="#00BFC4",col="#00BFC4") +
 
 spfolds_rand <- CreateSpacetimeFolds(pts_random_co,spacevar = "subregion",
                                      k=length(unique(pts_random_co$subregion)))
-dist_rand_sp <- plot_geodist(pts_random_co,co,
+dist_rand_sp <- plot_geodist(pts_random_co,co.ee,
                              sampling="Fibonacci", 
                              cvfolds= spfolds_rand$indexOut, 
                              unit="km",
@@ -155,8 +127,8 @@ dist_rand_sp$plot+scale_x_log10(labels=round)
 ## ----message = FALSE, warning=FALSE, results='hide'---------------------------
 
 
-nndmfolds_clstr <- nndm(pts_clustered, modeldomain=co, sampling = "Fibonacci",samplesize = 2000)
-dist_clstr <- plot_geodist(pts_clustered,co,
+nndmfolds_clstr <- nndm(pts_clustered, modeldomain=co.ee, samplesize = 2000)
+dist_clstr <- plot_geodist(pts_clustered,co.ee,
                            sampling = "Fibonacci",
                            cvfolds = nndmfolds_clstr$indx_test, 
                            cvtrain = nndmfolds_clstr$indx_train,
@@ -167,8 +139,8 @@ dist_clstr$plot+scale_x_log10(labels=round)
 
 ## ----message = FALSE, warning=FALSE, results='hide'---------------------------
 
-nndmfolds_rand <- nndm(pts_random_co,  modeldomain=co, sampling = "Fibonacci",samplesize = 2000)
-dist_rand <- plot_geodist(pts_random_co,co,
+nndmfolds_rand <- nndm(pts_random_co,  modeldomain=co.ee, samplesize = 2000)
+dist_rand <- plot_geodist(pts_random_co,co.ee,
                           sampling = "Fibonacci",
                           cvfolds = nndmfolds_rand$indx_test, 
                           cvtrain = nndmfolds_rand$indx_train,
